MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=1,NU=1
  
CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
	INTENT(IN)  :: T,Y,Z
	INTENT(OUT) :: DY
	
    DY(1) = 2D0*Z(1,1)/(1D0 + Z(1,1)**9.65D0) - Y(1)
    
    RETURN
  END SUBROUTINE DDES

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_1

! Example 4.4.2 in the SGT book.
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_1.M imports the data into
! Matlab and plots it.

!%Example 5 of D.R. Wille' and C.T.H. Baker, DELSOL--a numerical code for the
!% solution of systems of delay-differential equations, Appl. Numer. Math., 9
!% (1992) 223-234.  This is an example of a scalar equation that exhibits chaotic
!% behavior. Equation (7) of the paper includes a parameter gamma, but no value
!% is specified in the example.  A value of -1 gives qualitative agreement with
!% Fig. 3 of the paper.  The length of the interval of integration is also not
!% given.  Here an interval [0, 100] is used; it is clearly shorter than that of
!% the figure, but the qualitative behavior is seen already.

!%D.R. Will and C.T.H. Baker, DELSOL - a numerical code for the solution of systems of delay-differential equations, Appl. %Numer. Math., 9 (1992) 223-234.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the array NVAR.

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  ! Constant delay
  DOUBLE PRECISION, DIMENSION(NU) :: DELAY=(/ 2D0 /)
  ! Constant history
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/ 0.5D0 /)
  TYPE(DDVERK_SOL) :: SOL
  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: Y,YLAG

  DOUBLE PRECISION :: T0=0D0,TFINAL=100D0
  ! Prepare for output:
  INTEGER, PARAMETER :: NOUT=1000	
  DOUBLE PRECISION, DIMENSION(NOUT) :: T

  ! Local variable:
  INTEGER :: I

  OPTS = DDVERK_SET(RE=1D-6,AE=1D-6,INTERPOLATION=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,DELAY,HISTORY, &
                  (/ T0,TFINAL /),OPTIONS=OPTS)

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

    ! Form values for phase plane plot:
    T = (/ (DELAY + (I-1)*((TFINAL - DELAY)/(NOUT-1)), I=1,NOUT) /)
    Y = DDVERK_VAL(T,SOL)
    YLAG = DDVERK_VAL(T - DELAY(1),SOL)
  
    ! Write the solution to a file for subsequent plotting
    ! in Matlab.
    OPEN(UNIT=6, FILE='Example_1.dat')
    DO I = 1,NOUT
        WRITE(UNIT=6,FMT='(2D12.4)') Y%YT(I,1),YLAG%YT(I,1)
    ENDDO
      
    PRINT *,' Normal return from DDVERK90 with results'
    PRINT *," written to the file 'Example_1.dat'."
    PRINT *,' '
    PRINT *,' These results can be accessed in Matlab and'
    PRINT *,' y(t) plotted against ylag = y(t - delay) by'
    PRINT *,' '
    PRINT *," >> [y,ylag] = Example_1;"
    PRINT *,' '

	CALL DDVERK_PRINT_STATS(SOL)
  
  ELSE
  
    PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
    SOL%FLAG
  
  ENDIF
  
  STOP
END PROGRAM Example_1
