MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 3,NU=2

  DOUBLE PRECISION, PARAMETER :: s0=0.0031D0,T1=6.D0,gamma=0.001D0,          &
								Q=0.0275D0,k=2.8D0, a=6570.D0, BK=0.0382D0,  &
								r=6.96D0

 !DOUBLE PRECISION, PARAMETER :: s0=0.00372D0,T1=3.D0,gamma=0.1D0,          &
!								Q=0.00178D0,k=6.65D0, a=15600.D0, BK=0.0382D0,  &
!								r=6.96D0

CONTAINS

  FUNCTION f(y) RESULT(R1)
	IMPLICIT NONE
	DOUBLE PRECISION :: y,R1
	R1=a/(1+BK*y**r)
  END FUNCTION f

  SUBROUTINE DDES(T,Y,Z,DY)
	IMPLICIT NONE
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY

    DY(1) = s0*Z(2,1)-gamma*Y(1)-Q
    DY(2) = f(Y(1))-k*Y(2)
    DY(3) = 1-Q*exp(gamma*Y(3))/(s0*Z(2,2))
	   

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y)
	IMPLICIT NONE
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	INTENT(IN)  :: T
	INTENT(OUT) :: Y
      Y(1) = 3.325D0
	  IF(T < -T1) THEN
	      Y(2) = 9.5D0
	  ELSE
	      Y(2) = 10.D0
	  END IF
	  Y(3) = 120.D0
!      Y(1) = 3.5D0
!      Y(2) = 10.D0
!	  Y(3) = 50.D0
    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE BETA(T,Y,BVAL)
	IMPLICIT NONE
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NU) :: BVAL
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: BVAL

      BVAL(1) = T-T1
	  BVAL(2) = T-T1-Y(3)

    RETURN
  END SUBROUTINE BETA

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_10

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_10.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 0.D0,TFINAL=300.D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J


  OPTS = DDVERK_SET(RE=1D-6,AE=1D-6,INTERPOLATION=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,BETA,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_10.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(4D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_10.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_10;"
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_10
