MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 2,MU=1
  DOUBLE PRECISION ,PARAMETER :: kM1=1.34D0,kM2=1.6D9,kM3=8.D3, &
								 kM4=4.D7,kM5=1.D0,fr=1.D0,A=6.D-2, &
								 B=6.D-2,TAU=0.15D0

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY

    DY(1) = kM1*A*Y(2)-kM2*Y(1)*Z(2,1)+kM3*B*Y(1)-2*kM4*Y(1)**2
	DY(2) = -kM1*A*Y(2)-kM2*Y(1)*Z(2,1)+fr*kM3*B*Y(1)

    RETURN
  END SUBROUTINE DDES


END MODULE define_DDEs

!******************************************************************

PROGRAM Example_13

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_13.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   MU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/MU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 0.0D0,TFINAL=100.5D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=10000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J

  DOUBLE PRECISION, DIMENSION(MU) :: LAGS= (/ TAU/)
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/ 1.D-10,1.D-5/)

  OPTS = DDVERK_SET(RE=1D-9,AE=1D-18,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(TOL=1D-15,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-9,INTERPOLATION=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     ! Form values for phase plane plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_13.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(3D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_13.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_13;"
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

	 PRINT *,'T=',SOL%T(SOL%NPTS),' Y1=',SOL%Y(SOL%NPTS,1),' Y2=',SOL%Y(SOL%NPTS,2)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_13
