MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 3,MU=1
  DOUBLE PRECISION,PARAMETER :: TAU=3.D-2

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY

    DY(1) = -0.04D0*Y(1)+1.D4*Z(2,1)*Y(3)
	DY(2) = 0.04D0*Y(1)-1.D4*Z(2,1)*Y(3)-3.D7*Y(2)**2
	DY(3) = 3.D7*Y(2)**2

    RETURN
  END SUBROUTINE DDES


END MODULE define_DDEs

!******************************************************************

PROGRAM Example_16

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_16.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   MU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/MU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 0.D0,TFINAL=12.D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J
  CHARACTER(20) :: FNAME

  DOUBLE PRECISION, DIMENSION(MU) :: LAGS= (/ TAU/)
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/ 1.D0 ,0.D0,0.D0/)

  OPTS = DDVERK_SET(RE=1D-9,AE=1D-14,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(TOL=1D-9,INTERPOLATION=.TRUE.)!,SCALE=1.D2)
  !OPTS = DDVERK_SET(ABS_TOL=1D-10,REL_WEIGHTS=(/1.D3,1.D0,1.D3/), &
!					INTERPOLATION=.TRUE.)!,SCALE=1.D2)

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     ! Form values for phase plane plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
	 IF(TAU==1.D-2) THEN
	     FNAME='Example_16_1.dat'
	 ELSE
	     FNAME='Example_16_3.dat'
	 END IF
     OPEN(UNIT=6, FILE=FNAME)
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(4D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_16.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_16;"
     PRINT *,' '

	 PRINT *,'T=',SOL%T(SOL%NPTS),' Y1=',SOL%Y(SOL%NPTS,1),' Y2=',SOL%Y(SOL%NPTS,2) ,&
				' Y3=',SOL%Y(SOL%NPTS,3)

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_16
