MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 1,NU=1


CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY

    DY(1) = (1+exp(-T)) * Z(1,1) * exp(exp(-T+exp(-T)))
	   

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y,DERIVATIVE)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	LOGICAL :: DERIVATIVE
	INTENT(IN)  :: T,DERIVATIVE
	INTENT(OUT) :: Y
	IF(.NOT. DERIVATIVE) THEN
      Y(1) = exp(T-exp(-T))
	ELSE
      Y(1) = 0.D0
	END IF

    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE BETA(T,Y,BVAL)
    
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NU) :: BVAL
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: BVAL

      BVAL(1) = T-exp(-T)

    RETURN
  END SUBROUTINE BETA

  SUBROUTINE CHECK(T,Y,ERROR)
    
    DOUBLE PRECISION :: T,Y
    DOUBLE PRECISION :: ERROR
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: ERROR

      ERROR = ABS(Y - exp(T-exp(-T)))/exp(T-exp(-T))

    RETURN
  END SUBROUTINE CHECK

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_2

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_2.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 1.0D0,TFINAL=30.0D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  DOUBLE PRECISION :: ERROR,MAXERROR

  ! Local variables:
  INTEGER :: I


  OPTS = DDVERK_SET(RE=1D-6,AE=1D-6,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-9,INTERPOLATION=.TRUE.)
  
  SOL = DDVERK90(NVAR,NLAGS,DDES,BETA,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

      TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_2.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(2D12.4)') TINT(I),YINT%YT(I,1)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_2.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_2;"
     PRINT *,' '

	 MAXERROR = 0D0
     DO I = 1,SOL%NPTS
        CALL CHECK(SOL%T(I),SOL%Y(I,1),ERROR)
        MAXERROR = MAX(ERROR,MAXERROR)
     END DO
     PRINT *,' '
     PRINT *,' Comparison to the analytical solution shows'
	 PRINT *,' that at all steps the absolute error is less'
	 PRINT *,' than ',MAXERROR,'.'
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_2
