MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=1,NU=1

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z

      DY(1) = Y(1) * Z(1,1) / T

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE BETA(T,Y,BVAL)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NU) :: BVAL

      BVAL(1) = LOG(Y(1))

    RETURN
  END SUBROUTINE BETA

  SUBROUTINE CHECK(T,Y,ERROR)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION :: E1, E2, E3, EXACT, ERROR

      E1 = EXP(1D0)
      E2 = EXP(2D0)
      E3 = EXP(3D0 - EXP(1D0 - E1))
      IF (T <= E1) THEN
         EXACT = T
      ELSE
         IF (T <= E2) THEN
            EXACT = EXP(T/E1)
         ELSE
           IF (T <= E3) THEN
              EXACT = (E1/(3D0 - LOG(T)))**E1
           ELSE
              PRINT *, ' Solution not known for T > E3.'
              EXACT = Y(1)
           ENDIF
         ENDIF
      ENDIF
      ERROR = ABS((Y(1) - EXACT)/EXACT)

    RETURN
  END SUBROUTINE CHECK
END MODULE define_DDEs

!******************************************************************

PROGRAM Example_3 

! For t > 1, solve the DDE  y'(t) = y(t)*y(ln(y(t)))/t for t >= 1,
!                           y(t) = 1 for t <= 1.       
!
! The DDE is defined in the module define_DDEs. The problem is
! solved here with DDVERK90 and its output written to a file.
! Using the auxilary function getplotdata, it is then easy to
! import the data into Matlab and plot it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so
  ! that they can be used for dimensioning arrays here.
  ! They are passed to the solver in the array NVAR.

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  TYPE(DDVERK_SOL) :: SOL ! Solution in Matlab style. 
  ! The fields of SOL are expressed in terms of the
  ! number of differential equations, NEQN, and the 
  ! number of output points, NPTS:

  !   SOL%NPTS         -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)      -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 

  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT

  ! Local variables:
  INTEGER :: I,J
  DOUBLE PRECISION :: T0= 1.0D0,TFINAL=10.0D0
 ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  DOUBLE PRECISION :: ERROR,MAXERROR

  CHARACTER(7+6*NEQN) :: EXPORT
  ! FNAME is the name of the output file.
  CHARACTER(10) :: FNAME='Example_3.dat'  

  OPTS = DDVERK_SET(RE=1D-5,AE=1D-5,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-13)

  SOL = DDVERK90(NVAR,NLAGS,DDES,BETA,(/ 1D0 /),&
                  (/ T0, TFINAL /),OPTIONS=OPTS)

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN
 
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_3.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(2D12.4)') TINT(I),YINT%YT(I,1)
     END DO
      
     PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file '",FNAME,"'."
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' from the folder holding this file by'
     PRINT *,' '
     PRINT *," >> [t,y] = getplotdata('",FNAME,"');"
     PRINT *,' '
     PRINT *,' and then plotted with, e.g., '
     PRINT *,' '
     PRINT *,' >> plot(t,y)'
     PRINT *,' '

     MAXERROR = 0D0
     DO I = 1,SOL%NPTS
        CALL CHECK(SOL%T(I),SOL%Y(I,:),ERROR)
        MAXERROR = MAX(ERROR,MAXERROR)
     ENDDO
     PRINT *,' '
     PRINT *,' Comparison to the analytical solution shows that'
     PRINT *,' at all steps the error is less than ',MAXERROR
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE

     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG

  ENDIF
  
  STOP
END PROGRAM Example_3
