MODULE define_DDEs

  IMPLICIT NONE

  INTEGER, PARAMETER :: NEQN=4,NU=2

  ! Physical parameters assigned in the main program.
  DOUBLE PRECISION :: tau,omega

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
	INTENT(IN)  :: T,Y,Z
	INTENT(OUT) :: DY
	   
    ! Physical parameters
    DOUBLE PRECISION :: A=0.330D0,d=0.006D0,lambda=0.308D0,&
	gamma=0.040D0,epsilon=0.060D0

    ! Local variables
    DOUBLE PRECISION :: S,E,I,R,Noft,Itau,Somega,Eomega,&
    Iomega,Romega,Nomega,dSdt,dEdt,dIdt,dRdt

    ! Local solution variables and delayed solution values:
    S = Y(1)
    E = Y(2)
    I = Y(3)
    R = Y(4)
    Itau   = Z(3,1)
    Somega = Z(1,2)
    Eomega = Z(2,2)
    Iomega = Z(3,2)
    Romega = Z(4,2)

    Noft = S + E + I + R
    Nomega = Somega + Eomega + Iomega + Romega

    ! Local derivatives:
    dSdt = A - d*S - lambda*((S*I)/Noft) + gamma*Itau*exp(-d*tau)
    dEdt = lambda*((S*I)/Noft) -                                 &
           lambda*((Somega*Iomega)/Nomega)* exp(-d*omega) - d*E
    dIdt = lambda*((Somega*Iomega)/Nomega)*exp(-d*omega) -       &
           (gamma+epsilon+d)*I
    dRdt = gamma*I - gamma*Itau*exp(-d*tau) - d*R

    ! Derivatives for the integrator:
	DY = (/ dSdt, dEdt, dIdt, dRdt /)

    RETURN
  END SUBROUTINE DDES

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_4 

! Example 4.4.1 in the SGT book.
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_4.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M
  

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the array NVAR.

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  ! Constant delays and history:
  DOUBLE PRECISION, DIMENSION(NU) :: DELAYS=&
  (/ 42D0, 0.15D0 /)
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY=&
  (/ 15D0, 0D0, 2D0, 3D0 /)

  TYPE(DDVERK_SOL) :: SOL
  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT

  !   SOL%NPTS         -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)      -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 

  INTEGER :: I,J ! Local variables

  DOUBLE PRECISION :: T0= 0.0D0,TFINAL=350.0D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  ! These physical parameters are global variables
  ! in define_DDEs.  Assign their values here.
  tau = 42D0
  omega = 0.15D0

  OPTS = DDVERK_SET(RE=1D-5,INTERPOLATION=.TRUE.)
  SOL = DDVERK90(NVAR,NLAGS,DDES,DELAYS,HISTORY,TSPAN=(/ T0, TFINAL /),OPTIONS=OPTS)

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_4.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(5D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO
      
     PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_4.dat'."
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_4;"
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE

     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG

  ENDIF
  
  STOP
END PROGRAM Example_4
