MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=1,NU=1,OMEGA=1

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)
   
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU+OMEGA) :: Z
	DOUBLE PRECISION, DIMENSION(NEQN,NU) :: YLAG
	DOUBLE PRECISION, DIMENSION(NEQN,OMEGA) :: DYLAG
	INTENT(IN)  :: T,Y,Z
	INTENT(OUT) :: DY
    ! NOTE:
    ! This is a neutral problem. Columns 1...NLAGS of the Z
    ! matrix contain the delayed solution values and columns
    ! NLAGS+1...2*NLAGS contain the delayed derivative values.
     
      YLAG  = Z(:,1:NU)
      DYLAG = Z(:,NU+1:NU+OMEGA)

      DY(1) = COS(T)*(1D0 + YLAG(1,1)) + 0.3D0*Y(1)*DYLAG(1,1)  &
        + 0.7D0*SIN(T)*COS( T*SIN(T)**2 ) - SIN( T+T*SIN(T)**2 )
                   
    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y,DERIVATIVE)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	LOGICAL :: DERIVATIVE
	INTENT(IN)  :: T,DERIVATIVE
	INTENT(OUT) :: Y
    ! NOTE:
    ! This is a neutral problem.  Return the history
    ! in column 1 of the Y matrix and the derivative
    ! of the history in column 2.
	IF(.NOT. DERIVATIVE) THEN
      Y(1) = SIN(T)
	ELSE
      Y(1) = COS(T)
	END IF

    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE BETA(T,Y,BVAL)
    
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NU+OMEGA) :: BVAL
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: BVAL

      BVAL(1) = T*Y(1)**2
	  BVAL(2) = BVAL(1)

    RETURN
  END SUBROUTINE BETA

  SUBROUTINE CHECK(T,Y,ERROR)
    
    DOUBLE PRECISION :: T,Y
    DOUBLE PRECISION :: ERROR
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: ERROR

      ERROR = ABS(Y - SIN(T))

    RETURN
  END SUBROUTINE CHECK

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_5 

! Program to solve the last example in the SGT book. This 
! is a neutral problem with a state-dependent vanishing delay.
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_5.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN = number of equations
  !   NLAGS = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the array NVAR.

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(2) :: NLAGS = (/NU,OMEGA/)

  TYPE(DDVERK_SOL) :: SOL 
  ! The fields of SOL are expressed in terms of the
  ! number of differential equations, NEQN, and the 
  ! number of output points, NPTS:

  !   SOL%NPTS      -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)   -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 

  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT

  ! Local variables:
  INTEGER :: I,J
  DOUBLE PRECISION :: T0= 0.D0,TFINAL
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  DOUBLE PRECISION :: ERROR,MAXERROR

  TFINAL= 2*ASIN(1D0)

  OPTS = DDVERK_SET(RE=1D-10,AE=1D-10,NEUTRAL=.TRUE.,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-14,NEUTRAL=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,BETA,HISTORY,&
                  (/T0,TFINAL/),OPTIONS=OPTS)

  ! Was the solver successful?
  IF(SOL%FLAG == 0) THEN

 
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_5.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(2D12.4)') TINT(I),YINT%YT(I,1)
     END DO


    PRINT *,' Normal return from DDVERK90 with results'
    PRINT *," written to the file 'Example_5.dat'."
    PRINT *,' '
    PRINT *,' These results can be accessed in Matlab'
    PRINT *,' and plotted by'
    PRINT *,' '
    PRINT *," >> [t,y] = Example_5;"
    PRINT *,' '
    
    MAXERROR = 0D0
    DO I = 1,SOL%NPTS
       CALL CHECK(SOL%T(I),SOL%Y(I,1),ERROR)
       MAXERROR = MAX(ERROR,MAXERROR)
    END DO
    PRINT *,' '
    PRINT *,' Comparison to the analytical solution shows'
	PRINT *,' that at all steps the absolute error is less'
	PRINT *,' than ',MAXERROR,'.'
    PRINT *,' '

    CALL DDVERK_PRINT_STATS(SOL)

  ELSE

    PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
    SOL%FLAG

  ENDIF
  
  STOP
END PROGRAM Example_5

