MODULE define_DDEs
  IMPLICIT NONE
  INTEGER, PARAMETER :: NEQN=3,NU=2,NEF=1
CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
    INTENT(IN) :: T,Y,Z
    INTENT(OUT) :: DY
    DY(1) = -Y(1) * Z(2,1) + Z(2,2)
    DY(2) =  Y(1) * Z(2,1) - Y(2)
    DY(3) =  Y(2) - Z(2,2)
    RETURN
  END SUBROUTINE DDES

  SUBROUTINE EF(T,Y,DY,Z,G)
     DOUBLE PRECISION :: T
     DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
     DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
     DOUBLE PRECISION, DIMENSION(NEF) :: G
     INTENT(IN) :: T,Y,DY,Z
     INTENT(OUT) :: G
     ! Locate extrema as points where the derivative
     ! vanishes.  The DIRECTION option is used to
     ! distinguish maxima and minima.
     G(1) = DY(2)
     RETURN
  END SUBROUTINE EF
END MODULE define_DDEs
!******************************************************************
PROGRAM Example_6 
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_6.M imports the data into
! Matlab and plots it.
  USE define_DDEs
  USE DDVERK90_M
  IMPLICIT NONE
  ! The quantities
  !   NEQN  = number of equations
  !   NU    = number of delays
  !   NEF   = number of event functions 
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS
  INTEGER, DIMENSION(2) :: NVAR = (/NEQN,NEF/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)
  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS
  DOUBLE PRECISION :: T0=0.0D0,TFINAL=55.0D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  ! Local variables:
  INTEGER :: I,J
  DOUBLE PRECISION, DIMENSION(NU) :: LAGS=(/ 1.0D0 ,10.0D0/)
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/ 5.0D0 ,0.1D0,1.0D0/)

  OPTS = DDVERK_SET(RE=1D-6,AE=1D-6,INTERPOLATION=.TRUE.,DIRECTION=(/-1 /))
  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS,EVENT_FCN=EF) 
  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN
     ! Form values for smooth plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
     YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_6.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(4D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO
     ! Write the extrema to a file for plotting.
     OPEN(UNIT=7,FILE='Example_6extr.dat')
     DO I = 1,SOL%NE
        WRITE(UNIT=7,FMT='(I10,2D12.4)') SOL%IE(I), &
                          SOL%TE(I),SOL%YE(I,2)
     END DO
     PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_6.dat' and the"
     PRINT *," extrema written to 'Example_6extr.dat'."
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y,te,ye,ie] = Example_6;"
     PRINT *,' '
     CALL DDVERK_PRINT_STATS(SOL)
  ELSE
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
             SOL%FLAG
  ENDIF
  STOP
END PROGRAM Example_6