MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=2,NU=1,NEF=2

  INTEGER :: STATE

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION :: YLAG
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
   
 !  Physical parameters
    DOUBLE PRECISION, PARAMETER :: gamma=0.248D0,beta=1D0, &
                                   A=0.75D0,omega=1.37D0

    YLAG = Z(1,1)

    DY(1) = Y(2)       
    DY(2) = SIN(Y(1)) - STATE*gamma*COS(Y(1)) - beta*YLAG &
              + A*SIN(omega*T + ASIN(gamma/A))

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE EF(T,Y,DY,Z,G)

     DOUBLE PRECISION :: T
     DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
     DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
     DOUBLE PRECISION, DIMENSION(NEF) :: G

     G = (/ Y(1), ABS(Y(1)) - ASIN(1D0) /)

     RETURN
  END SUBROUTINE EF

  SUBROUTINE CHNG(HAVE_EVENT,TEVENT,YEVENT,DYEVENT,HINIT, &
     DIRECTION,ISTERMINAL,QUIT,IS_CHANGED)

     LOGICAL ,DIMENSION(NEF):: HAVE_EVENT
     INTEGER, DIMENSION(NEF) :: DIRECTION
     DOUBLE PRECISION :: TEVENT,HINIT
     DOUBLE PRECISION, DIMENSION(NEQN) :: YEVENT,DYEVENT
     LOGICAL :: QUIT,IS_CHANGED
     LOGICAL, DIMENSION(NEF) :: ISTERMINAL
	 INTENT(IN) :: HAVE_EVENT,TEVENT
	 INTENT(INOUT) :: YEVENT,DYEVENT,HINIT,DIRECTION,&
	                  ISTERMINAL,QUIT,IS_CHANGED


       IF (HAVE_EVENT(1)) THEN
          ! Restart the integration with initial values
		  ! that correspond to a bounce of the suitcase.
          STATE = -STATE
          YEVENT(1) = 0.0D0
          YEVENT(2) = 0.913*YEVENT(2)
          DIRECTION(1) = - DIRECTION(1)
		  IS_CHANGED=.TRUE.
     ! ELSE
     !    The suitcase fell over, NEVENT = 2. The integration
	 !    could be terminated by QUIT = .TRUE., but this 
	 !    event is already a terminal event.
       ENDIF 

    RETURN
  END SUBROUTINE CHNG

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_7 

! Example 4.4.4 in the SGT book.

! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_7.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !   NEF   = number of event functions 
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the array NVAR.

  INTEGER, DIMENSION(2) :: NVAR = (/NEQN,NEF/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  TYPE(DDVERK_SOL) :: SOL 
  ! The fields of SOL are expressed in terms of the
  ! number of differential equations, NEQN, and the 
  ! number of output points, NPTS:

  !   SOL%NPTS         -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)      -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 
  
  ! When there is an event function, there are fields

  !   SOL%NE           -- NE, number of events.
  !   SOL%TE(NE)       -- locations of events
  !   SOL%YE(NE,NEQN)  -- values of solution at events
  !   SOL%IE(NE)       -- identifies which event occurred

  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT
  ! Local variables:
  INTEGER :: I,J

  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  DOUBLE PRECISION, PARAMETER :: T0=0D0,TFINAL=12D0
  DOUBLE PRECISION :: TEND
  ! Initialize global variable that governs the
  ! form of the DDEs.
  STATE = 1

  OPTS = DDVERK_SET(RE=1D-8,DIRECTION=(/ -1,0 /), &
                 ISTERMINAL=(/ .FALSE.,.TRUE. /) ,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-5,DIRECTION=(/ -1,0 /), &
   !              ISTERMINAL=(/ .FALSE.,.TRUE. /) )

  SOL = DDVERK90(NVAR,NLAGS,DDES,(/ 0.1D0 /),(/ 0D0,0D0 /), &
         TSPAN=(/T0,TFINAL/),OPTIONS=OPTS,EVENT_FCN=EF,CHANGE_FCN=CHNG) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN
 
     TEND=SOL%T(SOL%NPTS)
	 ! Form values for smooth plot:
     TINT = (/ (T0+(I-1)*((TEND-T0)/(NOUT-1)), I=1,NOUT) /)
     YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_7.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(3D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO
      
     PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_7.dat'."
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted in a phase plane by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_7;"
     PRINT *,' '
     PRINT *,' '

     PRINT *,' Kind of Event:'
     DO I = 1,SOL%NE
        IF(SOL%IE(I) == 1) THEN
           PRINT *,' A wheel hit the ground at',SOL%TE(I)
        ELSE
           PRINT *,' The suitcase fell over at',SOL%TE(I)
        END IF
     END DO
     PRINT *,' '
	 CALL DDVERK_PRINT_STATS(SOL)
  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG

 
  ENDIF
  
  STOP
END PROGRAM Example_7
