MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=4,NU=1,NEF=1

  INTEGER :: STATE

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,YLAG,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
   
    ! Physical parameters
    DOUBLE PRECISION, PARAMETER :: H1=2D0,H2=0.8D0,&
    H3=1D4,H4=0.17D0,H5=0.5D0,H6=3D2,H7=0.12D0,H8=8D0
    DOUBLE PRECISION :: XI

      YLAG = Z(:,1)

      IF (STATE==1) THEN
         XI = 1D0
      ELSE
         XI = (10D0/9D0)*(1D0 - Y(4))
      END IF

      DY(1) = (H1 - H2*Y(3))*Y(1)
      DY(2) = XI*H3*YLAG(3)*YLAG(1) - H5*(Y(2) - 1D0)
      DY(3) = H4*(Y(2) - Y(3)) - H8*Y(3)*Y(1)
      DY(4) = H6*Y(1) - H7*Y(4)

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y,DERIVATIVE)

     DOUBLE PRECISION :: T
     DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	 LOGICAL :: DERIVATIVE
	 IF(.NOT. DERIVATIVE) THEN
       Y(1) = MAX(0D0,T+1D-6)
       Y(2) = 1D0
       Y(3) = 1D0
       Y(4) = 0D0
	 ELSE
       Y(1) = 0D0
       Y(2) = 0D0
       Y(3) = 0D0
       Y(4) = 0D0
	 END IF

    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE EF(T,Y,DY,Z,G)

     DOUBLE PRECISION :: T
     DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
     DOUBLE PRECISION, DIMENSION(NEQN,NU) :: Z
     DOUBLE PRECISION, DIMENSION(NEF) :: G

       G(1) = Y(4) - 1D-1

     RETURN
  END SUBROUTINE EF

  SUBROUTINE CHNG(HAVE_EVENT,TEVENT,YEVENT,DYEVENT,HINIT, &
                  DIRECTION,ISTERMINAL,QUIT,IS_CHANGED)

    LOGICAL, DIMENSION(NEF):: HAVE_EVENT
    LOGICAL, DIMENSION(NEF) :: ISTERMINAL
    INTEGER, DIMENSION(NEF) :: DIRECTION
    DOUBLE PRECISION :: TEVENT,HINIT
    DOUBLE PRECISION, DIMENSION(NEQN) :: YEVENT,DYEVENT
    ! NOTE:  THE INPUT VALUE OF QUIT IS .FALSE., SO IT NEED BE
    !        SET ONLY IF THE INTEGRATION IS TO BE TERMINATED.
    LOGICAL :: QUIT,IS_CHANGED

      STATE = -STATE
      HINIT = 1D-7
	  IS_CHANGED=.TRUE.

    RETURN
  END SUBROUTINE CHNG

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_8 

  ! Marchuk immunology model, Exercise 4.8 in the book.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !   NEF   = number of event functions 
  !
  ! are defined in the module define_DDEs as PARAMETERs so
  ! that they can be used for dimensioning arrays here.
  ! They are passed to the solver in the array NVAR.

  INTEGER, DIMENSION(2) :: NVAR = (/NEQN,NEF/)
  INTEGER, DIMENSION(1) :: NLAGS = (/NU/)

  TYPE(DDVERK_SOL) :: SOL ! Solution in Matlab style. 
  ! The fields of SOL are expressed in terms of the
  ! number of differential equations, NEQN, and the 
  ! number of output points, NPTS:

  !   SOL%NPTS         -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)      -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 

  DOUBLE PRECISION, DIMENSION(NU) :: LAGS=(/ 0.5D0 /)

  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT

  DOUBLE PRECISION :: T0=0.0D0,TFINAL=60.0D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J

  CHARACTER(7+6*NEQN) :: EXPORT
  ! FNAME is the name of the output file.
  CHARACTER(10) :: FNAME='Example_8.dat'  

  STATE = 1

  OPTS = DDVERK_SET(RE=1D-5,AE=1D-8,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-5)

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY,TSPAN=(/T0,TFINAL/),&
           OPTIONS=OPTS,EVENT_FCN=EF,CHANGE_FCN=CHNG) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN
    
 
     ! Form values for smooth plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
     YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_8.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(5D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

     PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file '",FNAME,"'."
     PRINT *,' '
     PRINT *,' '
     PRINT *,' To access the output in Matlab and plot the '
     PRINT *,' solution components scaled as in Fig. 15.8 '
     PRINT *,' of the Hairer, Norsett, and Wanner book, use'
     PRINT *,' '
     PRINT *,' >> [t,y] = Example_8;'
     PRINT *,' '

     ! Print integration statistics:
     CALL DDVERK_PRINT_STATS(SOL)
  ELSE

     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG

  ENDIF
  
  STOP
END PROGRAM Example_8
